@@ -3,12 +3,18 @@ import requests
 import datetime
 import schedule
 import time
-
-import RPi.GPIO as GPIO
+from itertools import groupby
+from operator import itemgetter
 
 HALF_POWER_HOURS = 8
-ZERO_POWER_HOURS = 4
+ZERO_POWER_HOURS = 6
+MAX_ULTIMATE_LOWEST_PRICE_HOURS = 5
+MAX_CONSECUTIVE_ZERO_POWER_HOURS = 2
+ULTIMATE_LOWEST_PRICE = 0.03
+ULTIMATE_HIGHEST_PRICE = 0.35
 
+# Ultimate maximum number of zero power hours
+SAFE_ZERO_POWER_HOURS = 12
 
 class HourPrices:
     def __init__(self):
@@ -47,7 +53,6 @@ class HourPrices:
             number_of_hours_to_skip : number_of_hours_to_skip + number_of_hours_to_get
         ]
 
-    # Tää skip oli aika hieno
     def get_lowest_prices(self, number_of_hours_to_get, number_of_hours_to_skip):
         hour_prices = self.hour_prices
         hour_prices.sort(key=lambda x: x["Rank"])
@@ -96,33 +101,18 @@ class HeatControl:
         schedule.every().day.at("00:00").do(self.new_day)
 
     def new_day(self):
+
         self.set_prices_for_today()
+
         # Print half power hours
         print("Half power hours:")
         for hour in self.HalfPowerHours:
             print(str(hour["Hour"]) + " " + str(hour["PriceNoTax"]))
+
         # Print zero power hours
         print("Zero power hours:")
         for hour in self.ZeroPowerHours:
             print(str(hour["Hour"]) + " " + str(hour["PriceNoTax"]))
-        # Print average price
-        print("Average price: " + str(self.average_price))
-
-        # Print price difference between highest and lowest
-        print(
-            "Price difference between highest and lowest: "
-            + str(self.price_difference_between_highest_and_lowest)
-        )
-
-        # Print highest prices
-        print("Highest prices:")
-        for hour in self.highest_prices:
-            print(str(hour["Hour"]) + " " + str(hour["PriceNoTax"]))
-
-        # Print lowest prices
-        print("Lowest prices:")
-        for hour in self.lowest_prices:
-            print(str(hour["Hour"]) + " " + str(hour["PriceNoTax"]))
 
         # Print hour prices
         print("Hour prices:")
@@ -131,6 +121,8 @@ class HeatControl:
             print(str(hour["Hour"]) + " " + str(hour["PriceNoTax"]))
 
     def set_rasbperry_pi_gpio_pin(pin_number, state):
+        import RPi.GPIO as GPIO
+
         GPIO.setmode(GPIO.BCM)
         GPIO.setup(pin_number, GPIO.OUT)
         GPIO.output(pin_number, state)
@@ -147,35 +139,65 @@ class HeatControl:
     def set_heat_on(self):
         self.set_rasbperry_pi_gpio_pin(17, False)
         self.set_rasbperry_pi_gpio_pin(27, False)
+    
+    def get_number_of_consecutive_zero_power_hours(self, hour):
+        
+        # Calculate number of consecutive hours
+        hours = [h['Hour'] for h in self.ZeroPowerHours]
+
+        # Add next hour to test if it is consecutive
+        hours.append(hour['Hour'])
+        
+        hours.sort()
+
+        def max_consecutive(numbers):
+            max_count = 0
+            current_count = 1
+            for i in range(1, len(numbers)):
+                if numbers[i] == numbers[i-1] + 1:
+                    current_count += 1
+                else:
+                    max_count = max(max_count, current_count)
+                    current_count = 1
+            return max(max_count, current_count)
+
+        return max_consecutive(hours)
 
     def set_prices_for_today(self):
-        hour_prices = HourPrices()
-        self.hour_prices = hour_prices.hour_prices
 
-        highest_prices = hour_prices.get_highest_prices(
-            ZERO_POWER_HOURS + HALF_POWER_HOURS, 0
-        )
+        self.hour_prices = HourPrices().hour_prices
+
+        self.hour_prices.sort(key=lambda x: x["Rank"], reverse=True)
 
         self.ZeroPowerHours = []
         self.HalfPowerHours = []
+        add_to_zero_power_hours = False
+        
+        for hour in self.hour_prices:
 
-        highest_prices.sort(key=lambda x: x["Hour"])
+            if len(self.ZeroPowerHours) < ZERO_POWER_HOURS:
+                
+                consecutive_hours = self.get_number_of_consecutive_zero_power_hours(hour)
 
-        # Reorder zero power hours and half power hours so that there are no consecutive zero power hours
-        for hour in highest_prices:
+                if consecutive_hours <= MAX_CONSECUTIVE_ZERO_POWER_HOURS:
+                    add_to_zero_power_hours=True
 
+            # Add to ZeroPowerHours always if price is above ultimate max price
+            if hour["PriceWithTax"] > ULTIMATE_HIGHEST_PRICE:
+                add_to_zero_power_hours = True
+            
+            # Keep heating on if price is below ultimate min price
+            if hour["PriceWithTax"] < ULTIMATE_LOWEST_PRICE:
+                continue
 
-            if self.ZeroPowerHours: 
-                print("debug")
-                print(self.ZeroPowerHours[-1]["Hour"])
-                print(hour["Hour"] - 1)
+            if add_to_zero_power_hours:
+                
+                # Don't add more than SAFE_ZERO_POWER_HOURS
+                if len(self.ZeroPowerHours) < SAFE_ZERO_POWER_HOURS:
+                    self.ZeroPowerHours.append(hour)
+                add_to_zero_power_hours = False
 
-
-            if len(self.ZeroPowerHours) < ZERO_POWER_HOURS and (
-                not self.ZeroPowerHours or self.ZeroPowerHours[-1]["Hour"] != hour["Hour"] - 1
-            ):                       
-                self.ZeroPowerHours.append(hour)
-            else:
+            elif len(self.HalfPowerHours) < HALF_POWER_HOURS:
                 self.HalfPowerHours.append(hour)
 
     def set_heat(self):
@@ -210,4 +232,3 @@ def main():
 
 if __name__ == "__main__":
     main()
-    
\ No newline at end of file
